Package: graph
Title: graph: A package to handle graph data structures
Version: 1.84.1
Authors@R: c(
        person("R", "Gentleman", role = "aut"),
        person("Elizabeth", "Whalen", role="aut"),
        person("W", "Huber", role="aut"),
        person("S", "Falcon", role="aut"),
        person("Jeff", "Gentry", role="aut"),
        person("Paul", "Shannon", role="aut"),
        person("Halimat C.", "Atanda", role = "ctb",
                comment = "Converted 'MultiGraphClass' and 'GraphClass' vignettes from Sweave to RMarkdown / HTML."
        ),   
        person("Paul", "Villafuerte", role = "ctb",
                comment = "Converted vignettes from Sweave to RMarkdown / HTML."
        ),
        person("Aliyu Atiku", "Mustapha", role = "ctb",
                comment = "Converted 'Graph' vignette from Sweave to RMarkdown / HTML."
        ),
        person("Bioconductor Package Maintainer", role = "cre",
                email = "maintainer@bioconductor.org"
        ))
Description: A package that implements some simple graph handling
        capabilities.
License: Artistic-2.0
Depends: R (>= 2.10), methods, BiocGenerics (>= 0.13.11)
Imports: stats, stats4, utils
Suggests: SparseM (>= 0.36), XML, RBGL, RUnit, cluster, BiocStyle,
        knitr
Enhances: Rgraphviz
Collate: AllClasses.R AllGenerics.R bitarray.R buildDepGraph.R
        methods-graph.R graphNEL.R clustergraph.R NELhandler.R
        edgefunctions.R graphfunctions.R GXLformals.R gxlReader.R
        random.R write.tlp.R mat2graph.R settings.R zzz.R
        standardLabeling.R TODOT.R toDotWithRI.R methods-graphAM.R
        attrData.R reverseEdgeDirections.R nodes-methods.R
        methods-multiGraph.R MultiGraph.R methods-graphBAM.R
        graph-constructors.R
LazyLoad: yes
Packaged: 2025-01-02 22:34:00 UTC; biocbuild
biocViews: GraphAndNetwork
RoxygenNote: 7.2.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/graph
git_branch: RELEASE_3_20
git_last_commit: 41ac6e9
git_last_commit_date: 2024-12-30
Repository: Bioconductor 3.20
Date/Publication: 2025-01-02
NeedsCompilation: yes
Author: R Gentleman [aut],
  Elizabeth Whalen [aut],
  W Huber [aut],
  S Falcon [aut],
  Jeff Gentry [aut],
  Paul Shannon [aut],
  Halimat C. Atanda [ctb] (Converted 'MultiGraphClass' and 'GraphClass'
    vignettes from Sweave to RMarkdown / HTML.),
  Paul Villafuerte [ctb] (Converted vignettes from Sweave to RMarkdown /
    HTML.),
  Aliyu Atiku Mustapha [ctb] (Converted 'Graph' vignette from Sweave to
    RMarkdown / HTML.),
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
