\name{internal}
\alias{nullgraphID}
\title{Variables used for internal purposes}
\description{
  The \code{nullgraphID} variable is used to store a default
  identifier. This should not be used by users.
}
\author{Saikat DebRoy}
\keyword{internal}
